"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
const plugin_syntax_jsx_1 = __importDefault(require("@babel/plugin-syntax-jsx"));
const template_1 = __importDefault(require("@babel/template"));
const t = __importStar(require("@babel/types"));
const utils_1 = require("@compiled/utils");
const class_names_1 = require("./class-names");
const css_prop_1 = require("./css-prop");
const styled_1 = require("./styled");
const append_runtime_imports_1 = require("./utils/append-runtime-imports");
const cache_1 = require("./utils/cache");
const is_compiled_1 = require("./utils/is-compiled");
const normalize_props_usage_1 = require("./utils/normalize-props-usage");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const packageJson = require('../package.json');
const JSX_SOURCE_ANNOTATION_REGEX = /\*?\s*@jsxImportSource\s+([^\s]+)/;
const JSX_ANNOTATION_REGEX = /\*?\s*@jsx\s+([^\s]+)/;
const COMPILED_MODULE = '@compiled/react';
let globalCache;
exports.default = (0, helper_plugin_utils_1.declare)((api) => {
    api.assertVersion(7);
    return {
        name: packageJson.name,
        inherits: plugin_syntax_jsx_1.default,
        pre() {
            this.sheets = {};
            let cache;
            if (this.opts.cache === true) {
                globalCache = new cache_1.Cache();
                cache = globalCache;
            }
            else {
                cache = new cache_1.Cache();
            }
            cache.initialize(Object.assign(Object.assign({}, this.opts), { cache: !!this.opts.cache }));
            this.cache = cache;
            this.includedFiles = [];
            this.pathsToCleanup = [];
            this.pragma = {};
        },
        visitor: {
            Program: {
                enter(_, state) {
                    const { file } = state;
                    if (file.ast.comments) {
                        for (const comment of file.ast.comments) {
                            const jsxSourceMatches = JSX_SOURCE_ANNOTATION_REGEX.exec(comment.value);
                            const jsxMatches = JSX_ANNOTATION_REGEX.exec(comment.value);
                            if (jsxSourceMatches && jsxSourceMatches[1] === COMPILED_MODULE) {
                                // jsxImportSource pragma found - turn on CSS prop!
                                state.compiledImports = {};
                                state.pragma.jsxImportSource = true;
                            }
                            if (jsxMatches && jsxMatches[1] === 'jsx') {
                                state.pragma.jsx = true;
                            }
                        }
                    }
                },
                exit(path, state) {
                    var _a;
                    if (!state.compiledImports) {
                        return;
                    }
                    const { pragma, opts: { importReact: shouldImportReact = true }, } = state;
                    (0, utils_1.preserveLeadingComments)(path);
                    (0, append_runtime_imports_1.appendRuntimeImports)(path, state);
                    const hasPragma = pragma.jsxImportSource || pragma.jsx;
                    if (!hasPragma && shouldImportReact && !path.scope.getBinding('React')) {
                        // React is missing - add it in at the last moment!
                        path.unshiftContainer('body', template_1.default.ast(`import * as React from 'react'`));
                    }
                    if (state.compiledImports.styled && !path.scope.getBinding('forwardRef')) {
                        // forwardRef is missing - add it in at the last moment!
                        path.unshiftContainer('body', template_1.default.ast(`import { forwardRef } from 'react'`));
                    }
                    const filename = (0, path_1.basename)((_a = state.filename) !== null && _a !== void 0 ? _a : '') || 'File';
                    const version = process.env.TEST_PKG_VERSION || packageJson.version;
                    path.addComment('leading', ` ${filename} generated by ${packageJson.name} v${version} `);
                    // Add a line break after the comment
                    path.unshiftContainer('body', t.noop());
                    // Callback when included files have been added.
                    if (this.includedFiles.length && this.opts.onIncludedFiles) {
                        this.opts.onIncludedFiles((0, utils_1.unique)(this.includedFiles));
                    }
                    // Cleanup paths that have been marked.
                    state.pathsToCleanup.forEach((clean) => {
                        switch (clean.action) {
                            case 'remove': {
                                clean.path.remove();
                                return;
                            }
                            case 'replace': {
                                clean.path.replaceWith(t.nullLiteral());
                                return;
                            }
                            default:
                                return;
                        }
                    });
                },
            },
            ImportDeclaration(path, state) {
                if (path.node.source.value !== COMPILED_MODULE) {
                    return;
                }
                // The presence of the module enables CSS prop
                state.compiledImports = state.compiledImports || {};
                // Go through each import and enable each found API
                path.get('specifiers').forEach((specifier) => {
                    if (!state.compiledImports || !specifier.isImportSpecifier()) {
                        // Bail out early
                        return;
                    }
                    ['styled', 'ClassNames', 'css', 'keyframes'].forEach((apiName) => {
                        var _a, _b;
                        if (state.compiledImports &&
                            t.isIdentifier((_a = specifier.node) === null || _a === void 0 ? void 0 : _a.imported) &&
                            ((_b = specifier.node) === null || _b === void 0 ? void 0 : _b.imported.name) === apiName) {
                            // Enable the API with the local name
                            state.compiledImports[apiName] = specifier.node.local.name;
                            specifier.remove();
                        }
                    });
                });
                if (path.node.specifiers.length === 0) {
                    path.remove();
                }
            },
            'TaggedTemplateExpression|CallExpression'(path, state) {
                const hasStyles = (0, is_compiled_1.isCompiledCSSTaggedTemplateExpression)(path.node, state) ||
                    (0, is_compiled_1.isCompiledStyledTaggedTemplateExpression)(path.node, state) ||
                    (0, is_compiled_1.isCompiledCSSCallExpression)(path.node, state) ||
                    (0, is_compiled_1.isCompiledStyledCallExpression)(path.node, state);
                if (hasStyles) {
                    (0, normalize_props_usage_1.normalizePropsUsage)(path);
                }
                const isCompiledUtil = (0, is_compiled_1.isCompiledCSSTaggedTemplateExpression)(path.node, state) ||
                    (0, is_compiled_1.isCompiledKeyframesTaggedTemplateExpression)(path.node, state) ||
                    (0, is_compiled_1.isCompiledCSSCallExpression)(path.node, state) ||
                    (0, is_compiled_1.isCompiledKeyframesCallExpression)(path.node, state);
                if (isCompiledUtil) {
                    state.pathsToCleanup.push({ path, action: 'replace' });
                    return;
                }
                const isCompiledComponent = (0, is_compiled_1.isCompiledStyledTaggedTemplateExpression)(path.node, state) ||
                    (0, is_compiled_1.isCompiledStyledCallExpression)(path.node, state);
                if (isCompiledComponent) {
                    (0, styled_1.visitStyledPath)(path, { context: 'root', state, parentPath: path });
                    return;
                }
            },
            JSXElement(path, state) {
                var _a;
                if (!((_a = state.compiledImports) === null || _a === void 0 ? void 0 : _a.ClassNames)) {
                    return;
                }
                (0, class_names_1.visitClassNamesPath)(path, { context: 'root', state, parentPath: path });
            },
            JSXOpeningElement(path, state) {
                if (!state.compiledImports) {
                    return;
                }
                (0, css_prop_1.visitCssPropPath)(path, { context: 'root', state, parentPath: path });
            },
        },
    };
});
//# sourceMappingURL=babel-plugin.js.map