"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.objectPropertyToString = void 0;
const t = __importStar(require("@babel/types"));
const evaluate_expression_1 = require("./evaluate-expression");
const templateLiteralToString = (template, meta, expressionToString) => {
    let result = '';
    for (let i = 0; i < template.quasis.length; i += 1) {
        result += template.quasis[i].value.raw;
        if (i < template.expressions.length) {
            const expression = template.expressions[i];
            if (t.isTSType(expression)) {
                // Passed a type instead of a value
                // e.g. `${any}`
                throw new Error(`${template.type} has a type instead of a value`);
            }
            const evaluatedExpression = (0, evaluate_expression_1.evaluateExpression)(expression, meta);
            result += expressionToString(evaluatedExpression.value, evaluatedExpression.meta);
        }
    }
    return result;
};
const binaryExpressionToString = (expression, meta, expressionToString) => {
    const { left, right, operator } = expression;
    if (operator === '+' && t.isExpression(left)) {
        const leftValue = expressionToString(left, meta);
        const rightValue = expressionToString(right, meta);
        return `${leftValue}${rightValue}`;
    }
    throw new Error(`Cannot use ${operator} for string operation. Use + for string concatenation`);
};
const expressionToString = (expression, meta) => {
    // handles {'key-name': 'value'} or {1: 'value'}
    if (t.isStringLiteral(expression) || t.isNumericLiteral(expression)) {
        return String(expression.value);
    }
    // handles {[key]: 'value'}
    if (t.isIdentifier(expression)) {
        const evaluatedExpression = (0, evaluate_expression_1.evaluateExpression)(expression, meta);
        if (evaluatedExpression.value === expression) {
            throw new Error(`Cannot statically evaluate the value of "${expression.name}".`);
        }
        return expressionToString(evaluatedExpression.value, evaluatedExpression.meta);
    }
    // handles {[`key-${name}`]: 'value'}
    if (t.isTemplateLiteral(expression)) {
        return templateLiteralToString(expression, meta, expressionToString);
    }
    // handles {['key-' + name]: 'value'}
    if (t.isBinaryExpression(expression)) {
        return binaryExpressionToString(expression, meta, expressionToString);
    }
    throw new Error(`${expression.type} has no name.'`);
};
/**
 * Returns string output of a ObjectProperty's key
 *
 * @param prop ObjectProperty expression
 * @param meta Metadata
 */
const objectPropertyToString = (prop, meta) => {
    const { computed, key } = prop;
    // handles {key: 'value'}
    if (t.isIdentifier(key) && !computed) {
        return key.name;
    }
    return expressionToString(key, meta);
};
exports.objectPropertyToString = objectPropertyToString;
//# sourceMappingURL=object-property-to-string.js.map